<?php
// $Id: stf.tags.inc,v 1.1.2.1 2007/04/11 20:48:39 profix898 Exp $

function _reptag_stf_init($context) {
  return array(
  
    '#[\#]{5}(.*?)[\#]{1,5}#'     => '<h5>\\1</h5>',
    '#[\#]{4}(.*?)[\#]{1,4}#'     => '<h4>\\1</h4>',
    '#[\#]{3}(.*?)[\#]{1,3}#'     => '<h3>\\1</h3>',
    '#[\#]{2}(.*?)[\#]{1,2}#'     => '<h2>\\1</h2>',
    '#[\#]{1}(.*?)[\#]{1}#'       => '<h1>\\1</h1>',
    
    '#^[\s]*=\[[\s]*$#m'          => '<ul>',
    '#^[\s]*\]=[\s]*$#m'          => '</ul>',
    '#^[\s]*=pre\[[\s]*$#mi'      => '<pre>',
    '#^[\s]*\]pre=[\s]*$#mi'      => '</pre>',
    '#^[\s]*=code\[[\s]*$#mi'     => '<code>',
    '#^[\s]*\]code=[\s]*$#mi'     => '</code>',
    
    '#^[*]{4,}[\s]*$#m'           => '<hr />',
    '#^[_]{4,}[\s]*$#m'           => '<hr />',
    '#^[-]{4,}[\s]*$#m'           => '<hr />',
    
    '#\\\=#'                      => '<br />',
    
    '# [\*]{1,2}(.*?)[\*]{1,2} #' => '<strong>\\1</strong>',
    '# [_]{1,2}(.*?)[_]{1,2} #'   => '<i>\\1</i>',
    '# [\^]{1,2}(.*?)[\^]{1,2} #' => '<sup>\\1</sup>',
    '# [~]{1,2}(.*?)[~]{1,2} #'   => '<sub>\\1</sub>',
    
    '#\[(.*?)\]@\(www\.(.*?)\)#'  => '<a href="http://www.\\2">\\1</a>',
    '#\[(.*?)\]@\((.*?)\)#'       => '<a href="\\2">\\1</a>',    
    
    '#@\(www\.(.*?)\)#'           => '<a href="http://www.\\1">\\1</a>',
    '#@\((.*?)\)#'                => '<a href="\\1">\\1</a>',      
    
    '#\[(.*?)\]@\{(.*?)\}#'       => '<img src="\\2" alt="\\1" />',
    '#@\{(.*?)\}#'                => '<img src="\\1" alt="" />',
    
    '#[\\\]{1}\#([0-9a-f]*)#'     => '&#x\\1;'
    
  );
}

function _reptag_stf_info() {
  return array(t('Simple Text Formatting'), TRUE);
}

function _reptag_stf_help() {
  return array(
  
    '# header 1 #'                                                         =>  '<h1>header 1</h1>',
    '## header 2 ##'                                                       =>  '<h2>header 2</h2>',
    '### header 3 ###'                                                     =>  '<h3>header 3</h3>',
    '#### header 4 ####'                                                   =>  '<h4>header 4</h4>',
    '##### header 5 #####'                                                 =>  '<h5>header 5</h5>',
    '**strong**'                                                           =>  '<strong>strong</strong>',
    '__italic__'                                                           =>  '<i>italic</i>',
    'text^sup^'                                                            =>  'text<sup>sup</sup>',
    'text~sub~'                                                            =>  'text<sub>sub</sub>',
    '> blockquote'                                                         =>  '<blockquote>blockquote</blockquote>',
    '- list<br />- list<br /><i> ( use one of - / + / * )</i>'             =>  '<ul><li>list</li><li>list</li></ul>',
    '#. list<br />#. list<br /><i> ( use one of 1. / #. / #) / 1) )</i>'   =>  '<ol><li>list</li><li>list</li></ol>',
    '----<br /><i> ( use one of - / _ / * )</i>'                           =>  '<hr />',
    '| cell 1 | cell 2 | cell 3 |<br />| cell 1 | cell 2 | cell 3 |'       =>  '<table border="1"><tr><td>cell 1</td><td>cell 2</td><td>cell 3</td></tr><tr><td>cell 1</td><td>cell 2</td><td>cell 3</td></tr></table>',
    '[link text]@(http://www.google.com)'                                  =>  '<a href="http://www.google.com">link text</a>',
    '[link text]@(www.google.com)'                                         =>  '<a href="http://www.google.com">link text</a>',
    '@(http://www.google.com)'                                             =>  '<a href="http://www.google.com">http://www.google.com</a>',
    '@(www.google.com)'                                                    =>  '<a href="http://www.google.com">www.google.com</a>',
    '[image text]@{path/to/sample.png}'                                    =>  '<img src="'.drupal_get_path('module', 'reptag').'/tags/sample.png" alt="image text" />',
    '@{path/to/sample.png}'                                                =>  '<img src="'.drupal_get_path('module', 'reptag').'/tags/sample.png" alt="" />',
    '=[<br />block<br />]='                                                =>  '<ul>block</ul>',
    '=pre[<br />pre block<br />]pre='                                      =>  '<pre>pre block</pre>',
    '=code[<br />$var = test();<br />]code='                               =>  '<code>$var = test();</code>',
    'escape char \#169'                                                    =>  'escape char &copy;'
    
  );
}

function _reptag_stf_process($text, $tags, $context) {
  //unify nl, cr, paragraph, ...
  $text = str_replace('<p>', "\n=x[\n", $text);
  $text = str_replace('</p>', "\n]x=\n", $text);
  $text = str_replace(array('<br>', '<br />', '\r\n', '\r'), "\n", $text);
  $unbreak = array('#^=x\[\n(=[pre|code|x|]*\[)#m' => "\\1", '#^(\][pre|code|x|]*=)\n\]x=#m' => "\\1", '#[\n]{2,}#' => "\n");
  $text = preg_replace(array_keys($unbreak), array_values($unbreak), $text);
  //ident blockquotes
  $text = str_replace(array('&#62;', '&gt;'), '>', $text);
  $text = preg_replace_callback('#^[>]+[\s]*(.*?)\n#m', '_reptag_stf_blockquote', $text);
  //process format tags
  $text = preg_replace(array_keys($tags), array_values($tags), $text);
  //numbered item list
  $text = preg_replace('#\n([0-9\#]{1}[\.\)]{1}[\s]*.*?\n[0-9\#]{1})#m', "\n<ol>\n\\1", $text);
  $text = preg_replace('#\n([0-9\#]{1}[\.\)]{1}[\s]*.*?)\n([^0-9\#|\n])#m', "\n\\1\n</ol>\n\\2", $text);
  $text = preg_replace('#</ol>[\n]{0,1}<ol>#m', "", $text);
  $text = preg_replace_callback('#^[0-9\#]{1}[\.\)]{1}[\s]*(.*?)\n#m', '_reptag_stf_item_list', $text);
  //not numbered item list
  $text = preg_replace('#\n([\*\-\+]{1}[\s]*.*?\n[\*\-\+]{1})#m', "\n<ul>\n\\1", $text);
  $text = preg_replace('#\n([\*\-\+]{1}[\s]*.*?)\n([^\*\-\+|\n])#m', "\n\\1\n</ul>\n\\2", $text);
  $text = preg_replace('#</ul>[\n]{0,1}<ul>#m', "", $text);
  $text = preg_replace_callback('#^[\*\-\+]{1}[\s]*(.*?)\n#m', '_reptag_stf_item_list', $text);
  //item table
  $text = preg_replace('#\n\|([\s]*(.*?)[\s]*\|)[\s]*\n([\|]{1})#m', "\n<table>\n|\\1\n\\3", $text);
  $text = preg_replace('#\n\|([\s]*(.*?)[\s]*\|)[\s]*\n([^\||\n])#m', "\n|\\1\n</table>\n\\3", $text);
  $text = preg_replace_callback('#<table>(.*?)</table>#s', '_reptag_stf_item_table', $text);
  //add/restore linebreaks and cleanup
  $cleanup = array(
    '#</blockquote>[\n\s\t ]*<blockquote>[\n\s\t ]*(<blockquote>)#i' => "\n\\1",
    '#(</blockquote>)[\n\s\t ]*</blockquote>[\n\s\t]*<blockquote>#i' => "\\1\n",
    '#=x\[#m' => '<p>',
    '#\]x=#m' => '</p>',
    '#(\n){2,}#' => "\n"
  );
  $text = preg_replace(array_keys($cleanup), array_values($cleanup), $text);
  $text = _filter_autop($text);
  
  return $text;
}

function _reptag_stf_blockquote($matches) {
  $level = substr_count($matches[0], '>');
  for ($i = 0; $i < $level; $i++) {
    $tags .= "\n<blockquote>\n";
  }
  $output = $tags.$matches[1].str_replace("\n<blockquote>\n", "\n</blockquote>\n", $tags);
  return $output;
}

function _reptag_stf_item_list($matches) {
  return "  <li>".$matches[1]."</li>\n";
}

function _reptag_stf_item_table($matches) {
  $lines = explode("\n", trim($matches[1]));
  $output = "<table border=\"1\">\n";
  foreach ($lines as $line) {
    $output .= "  <tr>\n";
    $cells = explode('|', $line);
    foreach ($cells as $cell) {
      if (!empty($cell)) {
        $output .= "    <td>".trim($cell)."</td>\n";
      }
    }
    $output .= "  </tr>\n";
  }
  $output .= "</table>\n";
  
  return $output;
}

?>
